/********************************************************************/
/*                                                                  */
/* Program name: eventmsg.c                                         */
/*                                                                  */
/* Description: test reading event messages using MQBAG             */
/*                                                                  */
/* Setup:       queue manager must be enabled for performance       */
/*              events. ( PERFMEV(enabled) )                        */
/*              A test queue must be enabled for service events     */
/*              QSVCIEV(HIGH) + QSVCINT(120000)                     */
/*              **vcint time is milliseconds**                      */
/*                                                                  */
/*              and/or queue depth events                           */
/*              QDPHIEV(ENABLED)                                    */
/*              QDEPTHLO(20)                                        */
/*              QDEPTHHI(60)                                        */
/*              **high/low depths are % of MAXDEPTH**               */
/*                                                                  */
/* Syntax:      eventmsg queuename                                  */
/*              default queuename is SYSTEM.ADMIN.PERFM.EVENT       */
/********************************************************************/
/*                                                                  */
/* Results:                                                         */
/*                                                                  */
/*                                                                  */
/*   Performance Events:                                            */
/*     queue depth high:                                            */
/*       mqrc       MQRC_Q_DEPTH_HIGH                               */
/*       str attrs  MQCA_Q_MGR_NAME                                 */
/*                  MQCA_BASE_Q_NAME                                */
/*       int attrs  MQIA_TIME_SINCE_RESET                           */
/*                  MQIA_HIGH_Q_DEPTH                               */
/*                  MQIA_MSG_ENQ_COUNT (puts since last reset)      */
/*                  MQIA_MSG_DEQ_COUNT (gets since last reset)      */
/*                                                                  */
/*     queue depth low:                                             */
/*       mqrc       MQRC_Q_DEPTH_LOW                                */
/*       str attrs  MQCA_Q_MGR_NAME                                 */
/*                  MQCA_BASE_Q_NAME                                */
/*       int attrs  MQIA_TIME_SINCE_RESET                           */
/*                  MQIA_HIGH_Q_DEPTH                               */
/*                  MQIA_MSG_ENQ_COUNT                              */
/*                  MQIA_MSG_DEQ_COUNT                              */
/*                                                                  */
/*     queue full:                                                  */
/*       mqrc       MQRC_Q_FULL                                     */
/*       str attrs  MQCA_Q_MGR_NAME                                 */
/*                  MQCA_BASE_Q_NAME                                */
/*       int attrs  MQIA_TIME_SINCE_RESET                           */
/*                  MQIA_HIGH_Q_DEPTH                               */
/*                  MQIA_MSG_ENQ_COUNT                              */
/*                  MQIA_MSG_DEQ_COUNT                              */
/*                                                                  */
/*     queue service interval high:                                 */
/*       mqrc       MQRC_Q_SERVICE_INTERVAL_HIGH                    */
/*       str attrs  MQCA_Q_MGR_NAME                                 */
/*                  MQCA_BASE_Q_NAME                                */
/*       int attrs  MQIA_TIME_SINCE_RESET                           */
/*                  MQIA_HIGH_Q_DEPTH                               */
/*                  MQIA_MSG_ENQ_COUNT                              */
/*                  MQIA_MSG_DEQ_COUNT                              */
/*                                                                  */
/*     queue service interval ok:                                   */
/*       mqrc       MQRC_Q_SERVICE_INTERVAL_OK                      */
/*       str attrs  MQCA_Q_MGR_NAME                                 */
/*                  MQCA_BASE_Q_NAME                                */
/*       int attrs  MQIA_TIME_SINCE_RESET                           */
/*                  MQIA_HIGH_Q_DEPTH                               */
/*                  MQIA_MSG_ENQ_COUNT                              */
/*                  MQIA_MSG_DEQ_COUNT                              */
/*                                                                  */
/*                                                                  */
/*                                                                  */
/********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/********************************************************************/
/* includes for MQI                                                 */
/********************************************************************/
#include <cmqc.h>
#include <cmqcfc.h>
#include <cmqbc.h>

#define BUFFER_SIZE      256

void  CheckCallResult(char *callText, MQLONG cc, MQLONG rc);

int main(int argc, char **argv)
{
   /******************************************************************/
   /* Declare MQI structures needed                                  */
   /******************************************************************/
   MQOD     od  = {MQOD_DEFAULT};       /* Object Descriptor         */
   MQMD     md  = {MQMD_DEFAULT};       /* Message Descriptor        */
   MQGMO    gmo = {MQGMO_DEFAULT};      /* get message options       */
   MQHCONN  Hcon      = 0;              /* connection handle         */
   MQHOBJ   Hobj      = 0;              /* object handle             */
   MQLONG   O_options = 0;              /* MQOPEN options            */
   MQLONG   C_options = 0;              /* MQCLOSE options           */
   MQLONG   CompCode  = 0;              /* completion code           */
   MQLONG   OpenCode  = 0;              /* MQOPEN completion code    */
   MQLONG   Reason    = 0;              /* reason code               */
   MQLONG   CReason   = 0;              /* reason code for MQCONN    */
   MQLONG   itemValue = 0;              /* item value                */

   char     QMName[50];                 /* queue manager name        */
   MQCHAR   strData[BUFFER_SIZE]= "";   /* string data               */
   MQLONG   strDataLen          = sizeof(strData);

   MQHBAG   replyBag=MQHB_UNUSABLE_HBAG;/* reply bag                 */

   /******************************************************************/
   /* Connect to queue manager                                       */
   /******************************************************************/
   QMName[0] = 0;                      /* default queue manager      */
   if (argc > 2)
      strcpy(QMName, argv[2]);

   MQCONN(QMName, &Hcon, &CompCode, &CReason);  
   if (CompCode == MQCC_FAILED)
   {
      CheckCallResult("MQCONN", CompCode, CReason);
      exit(1);
   }

   /******************************************************************/
   /* Initialize object descriptor for subject queue                 */
   /******************************************************************/
   strcpy(od.ObjectName, "SYSTEM.ADMIN.PERFM.EVENT");
   if (argc > 1)
   {
      strcpy(od.ObjectName, argv[1]);
   }

   /******************************************************************/
   /* Open the event queue for input; exclusive or shared.  Use of   */
   /* the queue is controlled by the queue definition here           */
   /******************************************************************/


   O_options = MQOO_INPUT_AS_Q_DEF 
               + MQOO_FAIL_IF_QUIESCING; 
   MQOPEN(Hcon, &od, O_options, &Hobj, &CompCode, &Reason);    

   if (Reason != MQRC_NONE)
   {
      MQDISC(&Hcon, &CompCode, &Reason);
      CheckCallResult("MQOPEN", CompCode, Reason);         
      exit(2);
   }

   OpenCode = CompCode;
   printf("now monitoring queue %s\n", od.ObjectName);
   /******************************************************************/
   /*   Get messages from the message queue                          */
   /******************************************************************/
   if (Reason == MQRC_NONE)
   {
      mqCreateBag(MQCBO_COMMAND_BAG, &replyBag, &CompCode, &Reason);
      CheckCallResult("mqCreateBag", CompCode, Reason);         
   }

   gmo.Options = MQGMO_WAIT + MQGMO_CONVERT;
   gmo.WaitInterval = MQWI_UNLIMITED;

   while (Reason == MQRC_NONE)
   {
      memcpy(md.MsgId, MQMI_NONE, MQ_MSG_ID_LENGTH);
      memcpy(md.CorrelId, MQMI_NONE, MQ_MSG_ID_LENGTH);
      memcpy(md.Format, MQFMT_EVENT, MQ_FORMAT_LENGTH);

      mqGetBag(Hcon, Hobj, &md, &gmo, replyBag, &CompCode, &Reason);
      CheckCallResult("mqGetBag", CompCode, Reason);         
   
      /*********************************************************************/
      /* Check the compCode and reason                                     */
      /*********************************************************************/

      if (Reason == MQRC_NONE)
      {
         mqInquireInteger(replyBag, MQIASY_REASON, MQIND_NONE, &itemValue,
                          &CompCode, &Reason);
         CheckCallResult("mqInquireInteger", CompCode, Reason);         
         printf("\nEvent reason code:");
         switch (itemValue)
         {
            case MQRC_Q_DEPTH_HIGH:
               printf("    MQRC_Q_DEPTH_HIGH (%ld)\n", itemValue);
               break;
            case MQRC_Q_DEPTH_LOW:
               printf("    MQRC_Q_DEPTH_LOW (%ld)\n", itemValue);
               break;
            case MQRC_Q_FULL:
               printf("    MQRC_Q_FULL (%ld)\n", itemValue);
               break;
            case MQRC_Q_SERVICE_INTERVAL_HIGH:
               printf("    MQRC_Q_SERVICE_INTERVAL_HIGH (%ld)\n", itemValue);
               break;
            case MQRC_Q_SERVICE_INTERVAL_OK:
               printf("    MQRC_Q_SERVICE_INTERVAL_OK (%ld)\n", itemValue);
               break;
            default:
               printf("    unexpected code (%ld)\n", itemValue);
         }

      } /* endif */

      /*********************************************************************/
      /* Now print out any user selectors containing diagnostics           */
      /*********************************************************************/
      if (Reason == MQRC_NONE)
      {

         mqInquireString(replyBag, MQCA_BASE_Q_NAME, 0, MQ_Q_NAME_LENGTH, strData,
                         &strDataLen, NULL, &CompCode, &Reason);
         if (CompCode == MQCC_OK && Reason == MQRC_NONE)
         {
            mqTrim(MQ_Q_NAME_LENGTH, strData, strData, &CompCode, &Reason);
            printf("Queue name:           %s\n", strData);
         }

         mqInquireString(replyBag, MQCA_Q_MGR_NAME, 0, MQ_Q_MGR_NAME_LENGTH, strData,
                         &strDataLen, NULL, &CompCode, &Reason);
         if (CompCode == MQCC_OK && Reason == MQRC_NONE)
         {
            mqTrim(MQ_Q_MGR_NAME_LENGTH, strData, strData, &CompCode, &Reason);
            printf("Queue manager name:   %s\n", strData);
         }

         mqInquireInteger(replyBag, MQIA_TIME_SINCE_RESET, MQIND_NONE, &itemValue,
                          &CompCode, &Reason);
         if (CompCode == MQCC_OK && Reason == MQRC_NONE)
         {
            printf("Time since reset:     %ld\n", itemValue);
         }

         mqInquireInteger(replyBag, MQIA_HIGH_Q_DEPTH, MQIND_NONE, &itemValue,
                          &CompCode, &Reason);
         if (CompCode == MQCC_OK && Reason == MQRC_NONE)
         {
            printf("High queue depth:     %ld\n", itemValue);
         }

         mqInquireInteger(replyBag, MQIA_MSG_ENQ_COUNT, MQIND_NONE, &itemValue,
                          &CompCode, &Reason);
         if (CompCode == MQCC_OK && Reason == MQRC_NONE)
         {
            printf("Enqueue count:        %ld\n", itemValue);
         }

         mqInquireInteger(replyBag, MQIA_MSG_DEQ_COUNT, MQIND_NONE, &itemValue,
                          &CompCode, &Reason);
         if (CompCode == MQCC_OK && Reason == MQRC_NONE)
         {
            printf("Dequeue count:        %ld\n", itemValue);
         }
      } /* endif */

   } /* endwhile */

   /***************************************************************************/
   /* Delete the response bag if successfully created.                        */
   /***************************************************************************/
   if (replyBag != MQHB_UNUSABLE_HBAG)
   {
      mqDeleteBag(&replyBag, &CompCode, &Reason);
      CheckCallResult("Delete the response bag", CompCode, Reason);
   }


   /******************************************************************/
   /* close the event queue - if it was opened                       */
   /******************************************************************/
   if (OpenCode != MQCC_FAILED)
   {
      C_options = 0;     
      MQCLOSE(Hcon, &Hobj, C_options, &CompCode, &Reason);  
   }

   /******************************************************************/
   /* Disconnect from queue manager (unless previously connected)    */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
      MQDISC(&Hcon, &CompCode, &Reason); 

   }
}

void  CheckCallResult(char *callText, MQLONG cc, MQLONG rc)
{
   if (cc != MQCC_OK)
   {
      printf("%s failed: Completion Code = %d : Reason = %d\n", callText, cc, rc);
   }
}
